/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.common.core.flow;

import com.iohao.game.action.skeleton.core.CmdInfo;
import com.iohao.game.action.skeleton.core.commumication.BroadcastContext;
import com.iohao.game.action.skeleton.core.commumication.BrokerClientContext;
import com.iohao.game.action.skeleton.core.commumication.InvokeExternalModuleContext;
import com.iohao.game.action.skeleton.core.flow.FlowContext;
import com.iohao.game.action.skeleton.core.flow.attr.FlowAttr;
import com.iohao.game.action.skeleton.protocol.HeadMetadata;
import com.iohao.game.action.skeleton.protocol.RequestMessage;
import com.iohao.game.action.skeleton.protocol.ResponseMessage;
import com.iohao.game.action.skeleton.protocol.external.RequestCollectExternalMessage;
import com.iohao.game.action.skeleton.protocol.external.ResponseCollectExternalMessage;
import com.iohao.game.common.kit.ArrayKit;

import java.io.Serializable;
import java.util.Objects;

/**
 * 自定义 FlowContext
 * <pre>
 *
 * </pre>
 *
 * @author 渔民小镇
 * @date 2023-07-24
 */
public class MyFlowContext extends FlowContext {
    private MyAttachment attachment;

    public String getNickname() {
        initAttachment();

        if (attachment == null) {
            return null;
        }

        return attachment.nickname;
    }

    public MyAttachment getAttachment() {
        initAttachment();
        return attachment;
    }

    private void initAttachment() {
        if (Objects.isNull(attachment)) {

            byte[] attachmentData = this.getRequest().getHeadMetadata().getAttachmentData();
            if (ArrayKit.isEmpty(attachmentData)) {
                return;
            }

            this.attachment = this.getAttachment(MyAttachment.class);
        }
    }

    /**
     * 推送数据给玩家
     *
     * @param cmdInfo 路由
     * @param data    业务数据
     */
    public void broadcast(CmdInfo cmdInfo, Object data) {

        ResponseMessage responseMessage = createResponseMessage(cmdInfo, data);

        BrokerClientContext brokerClientContext = this.option(FlowAttr.brokerClientContext);
        BroadcastContext broadcastContext = brokerClientContext.getBroadcastContext();
        // 为了方便理解，这里加上 userId；
        broadcastContext.broadcast(responseMessage, getUserId());
    }

    public ResponseCollectExternalMessage invokeExternal(Serializable data, int bizCode) {
        HeadMetadata headMetadata = getHeadMetadata();
        // 得到游戏对外服 id
        int sourceClientId = headMetadata.getSourceClientId();

        var requestCollectExternalMessage = new RequestCollectExternalMessage()
                // 根据业务码，调用游戏对外服与业务码对应的业务实现类 （AttachmentDataExternalBizRegion）
                .setBizCode(bizCode)
                // 元信息
                .setData(data)
                // 指定游戏对外服
                .setSourceClientId(sourceClientId);

        BrokerClientContext brokerClientContext = this.option(FlowAttr.brokerClientContext);
        InvokeExternalModuleContext context = brokerClientContext.getInvokeExternalModuleContext();
        return context.invokeExternalModuleCollectMessage(requestCollectExternalMessage);
    }

    @Override
    protected RequestMessage createRequestMessage(CmdInfo cmdInfo, Object data) {

        HeadMetadata headMetadata = getHeadMetadata();

        RequestMessage requestMessage = super.createRequestMessage(cmdInfo, data);

        requestMessage.getHeadMetadata()
                .setEndPointClientId(headMetadata.getEndPointClientId())
                .setSourceClientId(headMetadata.getSourceClientId())
        ;

        return requestMessage;
    }

    private ResponseMessage createResponseMessage(CmdInfo cmdInfo, Object data) {
        HeadMetadata headMetadata = getHeadMetadata();
        /*
         * 创建一个 HeadMetadata，并使用原有的一些信息；
         * 在广播时，只会给 HeadMetadata 中指定的游戏对外服广播。
         */
        HeadMetadata headMetadataClone = headMetadata
                .cloneHeadMetadata()
                .setCmdInfo(cmdInfo)
                .setEndPointClientId(headMetadata.getEndPointClientId())
                .setSourceClientId(headMetadata.getSourceClientId());

        // 创建一个响应对象
        ResponseMessage responseMessage = new ResponseMessage();
        responseMessage.setHeadMetadata(headMetadataClone);
        responseMessage.setData(data);

        return responseMessage;
    }

    private HeadMetadata getHeadMetadata() {
        return this.getRequest().getHeadMetadata();
    }
}